@@ -6,6 +6,9 @@
 *
 * Copyright (C) 2003 by Roaring Penguin Software Inc.
 *
+* This program may be distributed under the terms of the GNU General
+* Public License, Version 2.
+*
 ***********************************************************************/
 
 #ifdef EMBED_PERL
@@ -23,7 +26,7 @@
 #define PERLPARSE_NUM_ARGS 6
 
 static PerlInterpreter *my_perl = NULL;
-extern void xs_init ();
+extern void xs_init (pTHX);
 
 void
 init_embedded_interpreter(int argc, char **argv, char **env)
@@ -81,15 +84,6 @@ make_embedded_interpreter(char const *progPath,
 	perl_destruct(my_perl);
 	perl_free(my_perl);
 	my_perl = NULL;
-#ifdef PERL_SET_CONTEXT
-	PERL_SET_CONTEXT(NULL);
-#endif
-/* The next line should NOT be necessary, but perl 5.10 appears to have
-   a bug as described at http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=516913
-*/
-#ifdef PERL_SET_INTERP
-	PERL_SET_INTERP(NULL);
-#endif
 #else
 	syslog(LOG_WARNING, "Cannot destroy and recreate a Perl interpreter safely on this platform.  Filter rules will NOT be reread.");
 	return 0;
