@@ -9,19 +9,13 @@
 * http://www.roaringpenguin.com
 *
 * This program may be distributed under the terms of the GNU General
-* Public License, Version 2, or (at your option) any later version.
+* Public License, Version 2.
 *
 * This program was derived from the sample mail filter included in
 * libmilter/README in the Sendmail 8.11 distribution.
 ***********************************************************************/
 
-#define _BSD_SOURCE 1
-
-#ifdef __linux__
-/* On Linux, we need this defined to get fdopen.  On BSD, if we define
- * it, we don't get all the u_char, u_long, etc definitions.  GRR! */
-#define _POSIX_SOURCE 1
-#endif
+#define _DEFAULT_SOURCE 1
 
 #define MAX_ML_LINES 31
 
@@ -200,6 +194,8 @@ struct privdata {
     unsigned char filterFailed; /* Filter failed */
 };
 
+static void set_queueid(SMFICTX *ctx);
+
 static void append_macro_value(dynamic_buffer *dbuf,
 			       SMFICTX *ctx,
 			       char *macro);
@@ -693,10 +689,13 @@ mfconnect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *sa)
     }
 
 
+    /* Try grabbing the Queue ID */
+    set_queueid(ctx);
+
     if (doRelayCheck) {
 	char buf2[SMALLBUF];
 	int n = MXRelayOK(MultiplexorSocketName, buf2, data->hostip,
-			  data->hostname, data->hostport, data->myip, data->daemon_port);
+			  data->hostname, data->hostport, data->myip, data->daemon_port, data->qid);
 	if (n == MD_REJECT) {
 	    /* Can't call smfi_setreply from connect callback */
 	    /* set_dsn(ctx, buf2, 5); */
@@ -756,10 +755,14 @@ helo(SMFICTX *ctx, char *helohost)
 	data->heloArg = NULL;
     }
     data->heloArg = strdup_with_log(helohost);
+
+    /* Try grabbing the Queue ID */
+    set_queueid(ctx);
+
     if (doHeloCheck) {
 	char buf2[SMALLBUF];
 	int n = MXHeloOK(MultiplexorSocketName, buf2, data->hostip,
-			 data->hostname, data->heloArg, data->hostport, data->myip, data->daemon_port);
+			 data->hostname, data->heloArg, data->hostport, data->myip, data->daemon_port, data->qid);
 	if (n == MD_REJECT) {
 	    set_dsn(ctx, buf2, 5);
 	    cleanup(ctx);
@@ -811,7 +814,6 @@ envfrom(SMFICTX *ctx, char **from)
     int i;
     char buffer[SMALLBUF];
     char buf2[SMALLBUF];
-    char *queueid;
     char **macro;
     dynamic_buffer dbuf;
 
@@ -830,23 +832,8 @@ envfrom(SMFICTX *ctx, char **from)
 	return SMFIS_TEMPFAIL;
     }
 
-    /* Obtain the queue identifier */
-    queueid = smfi_getsymval(ctx, "i");
-
-    if (data->qid && data->qid != NOQUEUE) {
-	free(data->qid);
-	data->qid = NOQUEUE;
-	data->qid_written = 0;
-    }
-    if (queueid && *queueid) {
-	data->qid = strdup_with_log(queueid);
-	if (!data->qid) {
-	    data->qid = NOQUEUE;
-	    cleanup(ctx);
-	    DEBUG_EXIT("envfrom", "SMFIS_TEMPFAIL");
-	    return SMFIS_TEMPFAIL;
-	}
-    }
+    /* Set the Queue ID if it hasn't yet been set */
+    set_queueid(ctx);
 
     /* Copy sender */
     if (data->sender) {
@@ -963,8 +950,8 @@ envfrom(SMFICTX *ctx, char **from)
 	data->myip = NULL;
     }
 
-    if (queueid) {
-	append_mx_command(&dbuf, 'Q', queueid);
+    if (data->qid && data->qid != NOQUEUE) {
+	append_mx_command(&dbuf, 'Q', data->qid);
 	data->qid_written = 1;
     }
 
@@ -1192,27 +1179,21 @@ rcptto(SMFICTX *ctx, char **to)
 
     /* Apparently, Postfix offers an option to set the "i" macro at
        rcptto time */
-    if (!data->qid || (data->qid == NOQUEUE)) {
-	char *queueid = smfi_getsymval(ctx, "i");
-	if (queueid && *queueid) {
-	    if (!data->qid_written) {
-		/* Write this out separately; the write below may be skipped */
-		data->cmdFD = get_fd(data, "COMMANDS", data->cmdFD);
-		if (data->cmdFD >= 0) {
-		    dbuf_init(&dbuf);
-		    append_mx_command(&dbuf, 'Q', queueid);
-		    if (write_dbuf(&dbuf, data->cmdFD, data, "COMMANDS") >= 0) {
-			data->qid_written = 1;
-		    }
-		    dbuf_free(&dbuf);
-		    data->cmdFD = put_fd(data->cmdFD);
-		}
-	    }
-	    data->qid = strdup_with_log(queueid);
-	    if (!data->qid) {
-		data->qid = NOQUEUE;
-	    }
-	}
+    set_queueid(ctx);
+    if (data->qid && data->qid != NOQUEUE) {
+        if (!data->qid_written) {
+            /* Write this out separately; the write below may be skipped */
+            data->cmdFD = get_fd(data, "COMMANDS", data->cmdFD);
+            if (data->cmdFD >= 0) {
+                dbuf_init(&dbuf);
+                append_mx_command(&dbuf, 'Q', data->qid);
+                if (write_dbuf(&dbuf, data->cmdFD, data, "COMMANDS") >= 0) {
+                    data->qid_written = 1;
+                }
+                dbuf_free(&dbuf);
+                data->cmdFD = put_fd(data->cmdFD);
+            }
+        }
     }
 
     rcpt_mailer = smfi_getsymval(ctx, "{rcpt_mailer}");
@@ -1467,17 +1448,8 @@ eoh(SMFICTX *ctx)
 	return SMFIS_TEMPFAIL;
     }
 
-    /* If we don't have a qid yet, try to get one here.
-       Postfix only sets "i" at EOH, DATA and EOM */
-    if (!data->qid || (data->qid == NOQUEUE)) {
-	char *queueid = smfi_getsymval(ctx, "i");
-	if (queueid && *queueid) {
-	    data->qid = strdup_with_log(queueid);
-	    if (!data->qid) {
-		data->qid = NOQUEUE;
-	    }
-	}
-    }
+    /* Set the Queue ID if it hasn't yet been set */
+    set_queueid(ctx);
 
     /* We can close headerFD to save a descriptor */
     if (data->headerFD >= 0 && closefd(data->headerFD) < 0) {
@@ -1536,17 +1508,8 @@ body(SMFICTX *ctx, u_char *text, size_t len)
 	return SMFIS_TEMPFAIL;
     }
 
-    /* If we don't have a qid yet, try to get one here.
-       Postfix only sets "i" at EOH, DATA and EOM */
-    if (!data->qid || (data->qid == NOQUEUE)) {
-	char *queueid = smfi_getsymval(ctx, "i");
-	if (queueid && *queueid) {
-	    data->qid = strdup_with_log(queueid);
-	    if (!data->qid) {
-		data->qid = NOQUEUE;
-	    }
-	}
-    }
+    /* Set the Queue ID if it hasn't yet been set */
+    set_queueid(ctx);
 
     /* Write to file and scan body for suspicious characters */
     if (len) {
@@ -1671,17 +1634,8 @@ eom(SMFICTX *ctx)
 
     dbuf_init(&dbuf);
 
-    /* If we don't have a qid yet, try to get one here.
-       Postfix only sets "i" at EOH, DATA and EOM */
-    if (!data->qid || (data->qid == NOQUEUE)) {
-	char *queueid = smfi_getsymval(ctx, "i");
-	if (queueid && *queueid) {
-	    data->qid = strdup_with_log(queueid);
-	    if (!data->qid) {
-		data->qid = NOQUEUE;
-	    }
-	}
-    }
+    /* Set the Queue ID if it hasn't yet been set */
+    set_queueid(ctx);
 
     if (!data->qid_written && data->qid && (data->qid != NOQUEUE)) {
 	append_mx_command(&dbuf, 'Q', data->qid);
@@ -2652,11 +2606,14 @@ main(int argc, char **argv)
     /* Read key file if present */
     fp = fopen(KEY_FILE, "r");
     if (fp) {
-	fgets(ValidateHeader, sizeof(ValidateHeader), fp);
-	fclose(fp);
-	chomp(ValidateHeader);
+      if(fgets(ValidateHeader, sizeof(ValidateHeader), fp) != NULL) {
+        fclose(fp);
+        chomp(ValidateHeader);
+      } else {
+        ValidateHeader[0] = 0;
+      }
     } else {
-	ValidateHeader[0] = 0;
+      ValidateHeader[0] = 0;
     }
     if (smfi_register(filterDescriptor) == MI_FAILURE) {
 	fprintf(stderr, "%s: smfi_register failed\n", argv[0]);
@@ -3040,3 +2997,44 @@ write_dbuf(dynamic_buffer *dbuf,
 	   data->qid, DBUF_LEN(dbuf), filename, i);
     return -1;
 }
+
+/**********************************************************************
+*%FUNCTION: set_queueid
+*%ARGUMENTS:
+* ctx -- Sendmail filter mail context
+*%RETURNS:
+* Nothing.
+*%DESCRIPTION:
+* Obtains the Sendmail "i" macro and sets the privata data->qid
+* string to the value of the macro, if its value could be obtained.
+***********************************************************************/
+static void
+set_queueid(SMFICTX *ctx)
+{
+    struct privdata *data = DATA;
+    char const *queueid;
+
+    /* Get value of "i" macro */
+    queueid = smfi_getsymval(ctx, "i");
+
+    if (!queueid) {
+        /* Macro not set - nothing we can do.  */
+        return;
+    }
+
+    /* If qid is already set and is the same as what
+       we have, do nothing */
+    if (data->qid && !strcmp(data->qid, queueid)) {
+        return;
+    }
+
+    /* If qid is already set, free it */
+    if (data->qid && data->qid != NOQUEUE) {
+        free(data->qid);
+        data->qid_written = 0;
+    }
+    data->qid = strdup_with_log(queueid);
+    if (!data->qid) {
+        data->qid = NOQUEUE;
+    }
+}
