@@ -8,17 +8,11 @@
 * http://www.roaringpenguin.com
 *
 * This program may be distributed under the terms of the GNU General
-* Public License, Version 2, or (at your option) any later version.
+* Public License, Version 2.
 *
 ***********************************************************************/
 
-#define _BSD_SOURCE 1
-
-#ifdef __linux__
-/* On Linux, we need this defined to get fdopen.  On BSD, if we define
- * it, we don't get all the u_char, u_long, etc definitions.  GRR! */
-#define _POSIX_SOURCE 1
-#endif
+#define _DEFAULT_SOURCE 1
 
 #include "config.h"
 #include "mimedefang.h"
@@ -579,6 +573,7 @@ munch_mx_return(char *ans, char *msg, char const *qid)
 *  port -- client port
 *  myip -- My IP address, if known.
 *  daemon_port -- Listening port
+*  qid -- Queue ID
 * %RETURNS:
 *  1 if it's OK to accept connections from this host; 0 if not, -1 if error.
 *  If connection is rejected, error message *may* be set.
@@ -590,7 +585,8 @@ MXRelayOK(char const *sockname,
 	  char const *name,
 	  unsigned int port,
 	  char const *myip,
-	  unsigned int daemon_port)
+	  unsigned int daemon_port,
+          char const *qid)
 {
     char cmd[SMALLBUF];
     char ans[SMALLBUF];
@@ -613,7 +609,10 @@ MXRelayOK(char const *sockname,
 	myip = "UNKNOWN";
     }
 
-    if (percent_encode_command(1, cmd, sizeof(cmd), "relayok", ip, name, port_string, myip, daemon_port_string, NULL) < 0) {
+    if (!qid || !*qid) {
+        qid = "NOQUEUE";
+    }
+    if (percent_encode_command(1, cmd, sizeof(cmd), "relayok", ip, name, port_string, myip, daemon_port_string, qid, NULL) < 0) {
 	return MD_TEMPFAIL;
     }
     if (MXCommand(sockname, cmd, ans, SMALLBUF-1, NULL) < 0) return MD_TEMPFAIL;
@@ -631,6 +630,7 @@ MXRelayOK(char const *sockname,
 *  port -- client port
 *  myip -- My IP address, if known.
 *  daemon_port -- Listening port
+*  qid -- Queue ID
 * %RETURNS:
 *  1 if it's OK to accept messages from this sender; 0 if not, -1 if error or
 *  we should tempfail.
@@ -643,7 +643,8 @@ MXHeloOK(char const *sockname,
 	 char const *helo,
 	 unsigned int port,
 	 char const *myip,
-	 unsigned int daemon_port)
+	 unsigned int daemon_port,
+         char const *qid)
 {
     char cmd[SMALLBUF];
     char ans[SMALLBUF];
@@ -670,7 +671,11 @@ MXHeloOK(char const *sockname,
 	myip = "UNKNOWN";
     }
 
-    if (percent_encode_command(1, cmd, sizeof(cmd), "helook", ip, name, helo, port_string, myip, daemon_port_string, NULL) < 0) {
+    if (!qid || !*qid) {
+        qid = "NOQUEUE";
+    }
+
+    if (percent_encode_command(1, cmd, sizeof(cmd), "helook", ip, name, helo, port_string, myip, daemon_port_string, qid, NULL) < 0) {
 	return MD_TEMPFAIL;
     }
     if (MXCommand(sockname, cmd, ans, SMALLBUF-1, NULL) < 0) return MD_TEMPFAIL;
@@ -1323,7 +1328,8 @@ write_and_lock_pidfile(char const *pidfile, char *lockfile, int pidfile_fd)
 
     lockfile_fd = open(lockfile, O_RDWR|O_CREAT, 0666);
     if (lockfile_fd < 0) {
-	return -1;
+      free(lockfile);
+      return -1;
     }
 
     fl.l_type = F_WRLCK;
@@ -1332,8 +1338,9 @@ write_and_lock_pidfile(char const *pidfile, char *lockfile, int pidfile_fd)
     fl.l_len = 0;
 
     if (fcntl(lockfile_fd, F_SETLK, &fl) < 0) {
-	syslog(LOG_ERR, "Could not lock lockfile file %s: %m.  Is another copy running?", lockfile);
-	return -1;
+      syslog(LOG_ERR, "Could not lock lockfile file %s: %m.  Is another copy running?", lockfile);
+      free(lockfile);
+      return -1;
     }
     if (pidfile_fd >= 0) {
 	ftruncate(pidfile_fd, 0);
@@ -1342,9 +1349,11 @@ write_and_lock_pidfile(char const *pidfile, char *lockfile, int pidfile_fd)
 
 	/* Close the pidfile fd; no longer needed */
 	if (close(pidfile_fd) < 0) {
+      free(lockfile);
 	    return -1;
 	}
     }
+    free(lockfile);
 
     /* Do NOT close lockfile_fd... it will close and lock will be released
        when we exit */
